--[[
  Author: Atrophenus
  Version: 2.0.0
  Description:
    New Version of Share Target that now takes user input instead of having set commands.
    You can now share your target, location, or your next jump to active channel, a specific channel,
    group, guild, sector, or system. 

    Command:
    /share sharing chat destination
    
    Example:
    /share target private dude
    /share target channel 100
    /share target channel (This shares to active channel)
    /share target group
    /share location guild
    /share navigation group
]]

    ShareTarget = {}
    ShareTargetFunc = {}
    
    ShareDestinations = {
      'PRIVATE',
      'SECTOR',
      'CHANNEL',
      'SYSTEM',
      'GROUP',
      'GUILD',
    }
    
    ShareTypes = {
      'target',
      'location',
      'navigation'
    }
    
    -----------------------------------------------------------------------
    --                      Utility functions                            --
    -----------------------------------------------------------------------
    
    --Creates Rounding funciton
    function ShareTargetFunc.Rounding(number)
      if (number - math.floor(number) >= .5) then
        local newnum = math.ceil(number);
        return newnum;
      else 
        local newnum = math.floor(number);
        return newnum;
      end 
    end
    
    --Table counter to get the length of a table to simplify for loops
    function ShareTargetFunc.tablecounter(T)
      local count = 0
      for _ in pairs(T) do count = count + 1 end
      return count
    end
    
    
    --Checks user input against valid entries in the table
    function ShareTargetFunc.checker(chatlocation, table)
      local tablelength = ShareTargetFunc.tablecounter(table)
      local check = false
      for i = 1, tablelength do
        if chatlocation == table[i] then
          check = true
          break
        end
      end
      return check
    end
    
    
    
    
    
    -----------------------------------------------------------------------
    --                      Formating function                           --
    -----------------------------------------------------------------------
    --Returns information that you want to share based on user input this is seperate from the function
    --Called by the user command to keep code clean and easily editable
    function ShareTargetFunc.typeinfo(sharing, chatlocation)
      --empty string so i have something to return easily and i don't have to define local msg everytime
      local msg = ''
      -- Target sharing
      if sharing == ShareTypes[1] then
        local name, health, distance, factionid, guild, ship = GetTargetInfo()
        local location = ShortLocationStr(GetCurrentSectorid());
        local faction = FactionName[factionid]
        -- If the location is a public channel use this formatting
        if chatlocation == 'CHANNEL' then
          if name ~= nil then
            local distanceR = ShareTargetFunc.Rounding(distance)
              if health == nil then 
                msg = "Target: "..name.."; Distance: "..distanceR.."m away"
              else
                local health = ShareTargetFunc.Rounding(health * 100)
                local guildname = "["..guild.."]"..name
                if guild ~= "" then
                  msg = "Faction: "..faction.. "; Target: ".. guildname.. ";("..health.."%) Ship: ".. ship.."; Distance: "..distanceR.."m away."
                else
                  msg = "Faction: "..faction.. "; Target: ".. name.. ";("..health.."%) Ship: ".. ship.."; Distance: "..distanceR.."m away."
                end
              end
          end
        -- if you need a location tag in the same sector you must have a concussion
        elseif chatlocation == 'SECTOR' then
          if name ~= nil then
            local distanceR = ShareTargetFunc.Rounding(distance)
              if health == nil then 
                msg = "Target: "..name.."; Distance: "..distanceR.."m away"
              else
                local health = ShareTargetFunc.Rounding(health * 100)
                local guildname = "["..guild.."]"..name
                if guild ~= "" then
                  msg = "Faction: "..faction.. "; Target: ".. guildname.. ";("..health.."%) Ship: ".. ship.."; Distance: "..distanceR.."m away."
                else
                  msg = "Faction: "..faction.. "; Target: ".. name.. ";("..health.."%) Ship: ".. ship.."; Distance: "..distanceR.."m away."
                end
              end
          end
        --if the location is in System there is already a location tag attached
        elseif chatlocation == 'SYSTEM' then
          if name ~= nil then
            local distanceR = ShareTargetFunc.Rounding(distance)
              if health == nil then 
                msg = "Target: "..name.."; Distance: "..distanceR.."m away"
              else
                local health = ShareTargetFunc.Rounding(health * 100)
                local guildname = "["..guild.."]"..name
                if guild ~= "" then
                  msg = "Faction: "..faction.. "; Target: ".. guildname.. ";("..health.."%) Ship: ".. ship.."; Distance: "..distanceR.."m away."
                else
                  msg = "Faction: "..faction.. "; Target: ".. name.. ";("..health.."%) Ship: ".. ship.."; Distance: "..distanceR.."m away."
                end
              end
          end
        -- if the location isn't a public channel use this formatting
        else
          if name ~= nil then
            local distanceR = ShareTargetFunc.Rounding(distance)
              if health == nil then 
                msg = "In: "..location.."; Target: "..name.."; Distance:"..distanceR.."m away"
              else
                local health = ShareTargetFunc.Rounding(health * 100)
                local guildname = "["..guild.."]"..name
                if guild ~= "" then
                  msg = "In: "..location.."; Faction: "..faction.. "; Target: ".. guildname.. ";("..health.."%) Ship: ".. ship.."; Distance:"..distanceR.."m away."
                else
                  msg = "In: "..location.."; Faction: "..faction.. "; Target: ".. name.. ";("..health.."%) Ship: ".. ship.."; Distance: "..distanceR.."m away."
                end
              end
          end    
        end
      --location sharing
      elseif sharing == ShareTypes[2] then
        local systemid = GetCurrentSectorid();
        msg = 'Location: '..ShortLocationStr(systemid);
      
      --Navigation sharing
      elseif sharing == ShareTypes[3] then
        local systemid = GetCurrentSectorid();
        local nexthopid = NavRoute.GetNextHop();
        -- NavRoute.GetNextHop(); returns current sector id if no navigation is set
        if nexthopid ~= systemid then
          msg = 'Location: '..ShortLocationStr(systemid).."; Next Jump: "..ShortLocationStr(nexthopid)
        else
          msg = 'Location: '..ShortLocationStr(systemid)
        end
        --return the msg with the desired information
      end return msg
    end
    
    
    
    
    
    -----------------------------------------------------------------------
    --                         Main Function                             --
    -----------------------------------------------------------------------
    
    function ShareTarget.share(data, args) 
      -- What the user passed in to share
      local sharing = string.lower(args[1]);
    
      -- Where to share it too
      local chat = string.upper(args[2]);
      
      -- If it's to a person or a specific channel number
      local destination = args[3];
      
      -- Creates a local boolean to do checks with
      local validshare = ShareTargetFunc.checker(sharing, ShareTypes)
      local validchatlocation = ShareTargetFunc.checker(chat, ShareDestinations)
      
      -- Used to make sure if we are sharing to a channel number specifically it's an actual number
      -- By checking if it's nil as strings with anything other than numbers in them returns nil
      local channel = tonumber(destination)
      
      --Formats the information and puts it into a variable to call
      local info = ShareTargetFunc.typeinfo(sharing, chat)
    
      --Color for error prints
      local errorcolor = '\127FF0000'
    
      -- makes sure we aren't just sending an empty string
      if info ~= "" then
          -- If it's something this program can share
        if validshare == true then
          -- If it's a valid place to send something
          if validchatlocation == true then
            --If chat is is a private message 
            if chat == 'PRIVATE' and destination == nil then
              print(errorcolor..'**Please specify a player to share information to.')
            elseif chat == 'PRIVATE' then
              SendChat(info, chat, destination)
            --If we are sharing to a public channel and destination does not equal nil but the channel is not an actual number
            elseif chat == 'CHANNEL' and destination ~= nil and channel == nil then
              print(errorcolor..'**Please specify an actual channel to share information to.') 
            --If our chat is a public channel and they want to share it too a specific channel
            elseif chat == 'CHANNEL' and destination ~= nil then
              SendChat(info, chat, destination)
            --Else shares to an active channel or any other place that doesn't require a destination parameter
            else
              SendChat(info, chat)
            end
          else
            print(errorcolor..'**Please specify a valid chat to share too; private, sector, channel, system, group, or guild')
          end
        else
          print(errorcolor..'**Please specify target, location, or navigation for sharing')
        end
      end
    end
    
    
    RegisterUserCommand('share', ShareTarget.share)